"use strict";

exports.__esModule = true;
exports.shouldBeOneOf = exports.includes = exports.startsWith = exports.isNumber = exports.invariantVideo = exports.invariantImage = exports.invariant = void 0;

var invariantFor = function invariantFor(type) {
  return function (condition, parameter, value, message) {
    var source = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : "/image_transformation_reference#" + parameter + "_parameter";

    if (!condition) {
      value = typeof value === 'string' ? "'" + value + "'" : value;
      source = source ? "http://cloudinary.com/documentation" + source : null;
      throw new Error("Cloudinary " + type + ":: " + parameter + " " + message + ", received: " + value + (source ? " - see " + source : ''));
    }
  };
};

var invariant = invariantFor('');
exports.invariant = invariant;
var invariantImage = invariantFor('Image ');
exports.invariantImage = invariantImage;
var invariantVideo = invariantFor('Video ');
exports.invariantVideo = invariantVideo;

var isNumber = function isNumber(value) {
  return typeof value === 'number' || !Number.isNaN(Number(value));
};

exports.isNumber = isNumber;

var startsWith = function startsWith(needle, haystack) {
  return haystack.indexOf(needle) === 0;
};

exports.startsWith = startsWith;

var includes = function includes(needle, haystack) {
  return haystack.indexOf(needle) > -1;
};

exports.includes = includes;

var shouldBeOneOf = function shouldBeOneOf(possibleValues) {
  return "should be one of ['" + possibleValues.join("', '") + "']";
};

exports.shouldBeOneOf = shouldBeOneOf;